*-------------------------------------------------------------------
*
*    FUNCTION browse_x           
*    Beta test Freeware version 
*
*    (C) Copyright Lee Rodgers, 1993
*
*    Internet: leebert@aol.com
*   
*    SYNTAX:
*    ? browse_x( <expC1> ;   && window name
*                <expC2> ;   && window title
*                <expN1> ;   && top row
*                <expN2> ;   && top col
*                <expN3> ;   && end row
*                <expN4> ;   && end col
*                <expC3> ;   && current work area/ alias
*                <expC4> ;   && browse-for clause
*                <expC5> ;   && browse-field expression * each field must make a call to brx_FldWh()
*                <expC6> ;   && name of seek field
*                <expC7> ;   && array of button objects & associated meanings (remember to pass by reference)
*    
*
*   BROWSE STRING EXAMPLE:
*   s_fld_expressn = [ role.role_name :H="ROLE NAME (^home)" :W=brx_FldWh() ]
*
*   ARRAY example:
*   DIMENSION a_objects[ 3, 6 ]
*   a_objects[ 1, nSeekFld ] = "contact.LN" && the seek field
*   a_objects[ 1, nTagName ] = "LN"         && the index tag name
*   a_objects[ 1, nObjName ] = "LN_OBJ"     && an arbitrary name
*   a_objects[ 1, nUserDesc] = "Last Name"  && what the user sees
*   a_objects[ 1, nBrowFld ] = "LN"         && the browse field's name
*   a_objects[ 1, nFlag    ] = .F.          && a flag
*    
*-------------------------------------------------------------------
FUNCTION browse_x           
PARAMETERS s_wind_title, ;
           nRow1, nCol1, nRow2, nCol2, ;
           s_forclause, s_fld_expressn, ;
           s_seek_fld,  a_objects

*s_old_keycomp = SET("KEYCOMP")  && need to keep global of keycomp, SET("KEYCOMP") doesn't work

*---fix key compatibility
IF _windows
  SET KEYCOMP TO WINDOWS
  SET COLO OF SCHEME 10 TO n/w,w+/b,,,,gr+/r,W*/W*,,,
ELSE
  SET KEYCOMP TO DOS
  SET COLOR OF SCHEME 10 TO ;
     N/W,GR+/R,GR+/GR,GR+/W,N+/W,GR+/GR,W+/BG,B/N,GR+/W,N+/W,+
ENDIF


STORE .F. TO l_quitting, is_selected, l_rd_kill
STORE .T. TO l_go_jumpwind, l_hotkeys_off
STORE "" TO s_seek_str, s_old_seek
STORE "BUTTON" TO s_actiontype
n_recno       = 1
n_elaps_keytm = SECONDS()
n_order       = 1

*-----------------------------
*--array column name tokens
nSeekFld = 1
nTagName = 2
nObjName = 3
nUserDesc= 4
nBrowFld = 5
nFlag    = 6
*===========================================================


*----------------------------------------------------------
*---build buttons strings & vars
nButtnSize = 0        && init. button size var
s_buttons  = [ PICTURE "@*HN ]  && init. button picture string
n_numb_of_objs = ALEN( a_objects, 1 )

FOR _n = 1 TO n_numb_of_objs
  *-make sure browse field names are upper case
  a_objects[ _n, nBrowFld ] = UPPER( a_objects[ _n, nBrowFld ] )

  *--button string
  s_buttons = s_buttons + a_objects[ _n, nUserDesc ] + IIF( _n < n_numb_of_objs , [;] , ["] )

  *--max size of individ. buttons
  nButtnSize = IIF(LEN(a_objects[_n,nUserDesc])>nButtnSize, ;
                   LEN(a_objects[_n,nUserDesc]) , ;
                   nButtnSize ;
                  )
ENDFOR
*==========================================================

*--adjust size of window.....
*--take (max button size x # of objects) + (spaces x # of objs)
nButtnLen = (nButtnSize * n_numb_of_objs) + (3 * n_numb_of_objs)
n_get_col = IIF( nButtnLen < 40, (40/2)-(nButtnLen/2), 1 )
n_wind_width = IIF( nButtnLen >=40, nButtnLen + 14 + 2, 57 )


*----------------------------------------------------------
*---define control window
IF NOT WEXIST("br_control") 
 DEFINE WINDOW br_control ;
  FROM nRow1  , nCol1  ;
  TO   nRow1+4, n_wind_width ;
  TITLE "INCREMENTAL SEARCH CONTROLS" ;
  FLOAT ;
  CLOSE ;
  SHADOW ;
  NOMINIMIZE ;
  SYSTEM ;
  COLOR SCHEME 8
ENDIF
*==========================================================


*----------------------------------------------------------
*----define browse window
IF NOT WEXIST("BROWSE_X")
  DEFINE WINDOW BROWSE_X ;
    FROM nRow1+5, nCol1 ;
    TO   nRow2,   nCol2 ;
    HALFHEIGHT ;
    CLOSE ;
    FLOAT ;
    GROW ;
    ZOOM ;
    SYSTEM ;
    SHADOW ;
    COLOR SCHEME 10
		
  BROWSE WINDOW BROWSE_X ;
    TITLE s_wind_title ;
    NOREFRESH ;
    NOWAIT ;
    NOMENU ;
    NOEDIT ;
    NOAPPEND ;
    NODELETE ;
    FOR &s_forclause ;
    WHEN .F. ;
    FIELDS &s_fld_expressn ;
    COLOR SCHEME 10
    

  RELEASE WINDOW browse_x

ENDIF
*----------------------------------------------------------


*----------------------------------------------------------
*------@ SAY's & GET buttons
IF WVISIBLE("br_control")
	ACTIVATE WINDOW br_control SAME
ELSE
	ACTIVATE WINDOW br_control NOSHOW
ENDIF

@ 0,1 ;
    GET s_seek_str ;
	SIZE 1,40 ;
	DEFAULT " " ;
	PICTURE "@!" ;
	WHEN seek_str_WHEN() 
	
@ 2, n_get_col ;
    GET n_order ;
	&s_buttons ;
	SIZE 1, nButtnSize, 1 ;
	VALID brx_ord_valid( @a_objects, n_order )
	
@ 2, WCOLS()-15 ;
    GET nGobrow ;
	PICTURE "@*HN Browse Window" ;
	SIZE 1,13,1 ;
	DEFAULT 1 ;
	WHEN  nGobrow_when() ;
	VALID nGobrow_valid()
*==========================================================

IF NOT WVISIBLE("br_control")
	ACTIVATE WINDOW br_control
ENDIF

*----------------------------------------------------------
*---READ
READ CYCLE ;
	WHEN brx_rd_when() ;
	ACTIVATE brx_rd_act() ;
	DEACTIVATE brx_rd_deact() ;
    VALID brx_rd_valid() ;
	OBJECT (2 + n_numb_of_objs)
*========================================================


*----------------------------------------------------------
*--clean up code
IF ( !WVISIBLE("BR_CONTROL") OR !WVISIBLE(s_wind_title) ) ;
   OR INLIST( LASTKEY(), 27, 17, 23 ) 
  RELEASE WINDOWS br_control, (s_wind_title)
ENDIF

	
=map_keys(.F.)
=map_sel_keys(.F.)

*--SET KEYCOMP TO &s_old_KeyComp  && won't work

RETURN is_selected

*------------------------------------------------------------
*
*     FUNCTION nGobrow_when
*
*   CALLED BY: FUNCTION browse_x           
*
**------------------------------------------------------------
FUNCTION nGobrow_when
IF l_go_jumpwind
  KEYBOARD '{ENTER}'
ENDIF
l_go_jumpwind = .F.
RETURN

*----------------------------------------------------------
*
*  FUNCTION brx_FldWh
*
*   CALLED BY: FUNCTION browse_x
*
*----------------------------------------------------------
FUNCTION brx_FldWh
PARAMETERS s_type

IF INLIST( LASTKEY(), 3,5,18,24 ) && up,down,pgup,pgdn
  RETURN .T.
ENDIF

IF TYPE('s_type') # "L" AND !EMPTY(s_type)
  s_actiontype = s_type
ENDIF

ACTIVATE WINDOW (s_wind_title)

nSeekFld = 1
nTagName = 2
nObjName = 3
nUserDesc= 4
nBrowFld = 5
nFlag    = 6
isRightField = .F.

s_seek_str = ALLTRIM( s_seek_str )
=say_say()

IF s_actiontype = "HOTKEY"
   s_seek_str = ""
   SHOW GET s_seek_str
ENDIF

*----in case of ctrl+f1 bypass of validation of fields
IF l_hotkeys_off
 =map_keys(.T.)
 =map_sel_keys(.T.)
 l_hotkeys_off = .F.

 *--seek_it()'s 2nd parameter = .F. = NO READ TIMEOUT IN seek_it() f() call
 =seek_it( s_seek_str, .F. )  && can't do read timeout here 'cuz causes ghost window
 s_actiontype = "BUTTON"
ENDIF

*--event handler
DO CASE
 CASE s_actiontype = "BUTTON"

    *--if this is the browse field that belongs to the order object

    IF UPPER(a_objects[ n_order, nBrowFld ]) = UPPER(VARREAD())
      WAIT WINDOW NOWAIT a_objects[ n_order, nUserDesc ] + CHR(13) + " ORDER"
      s_actiontype = ""
      isRightField = .T.
      a_objects[ n_order, nFlag ] = .F.
    ELSE
      isRightField = .F.
      KEYBOARD '{END}'
      KEYBOARD '{END}'
    ENDIF

 CASE s_actiontype = "HOTKEY"

    *--is this a field that allows "ctrl+home" order setting by user?
    n_elem_numb = ASCAN( a_objects, UPPER(VARREAD()))
    
    *--if this browse field is a valid "ctrl+home" field
    IF n_elem_numb > 0
       n_obj_2_use = ASUBSCRIPT( a_objects, n_elem_numb, 1 )
       s_actiontype = "BUTTON"
       ACTIVATE WINDOW br_control
       _curobj = 1 + n_obj_2_use    && _curobj = 1 = OBJNUM( s_seek_str )
       KEYBOARD "{ENTER}"
       l_go_jumpwind = .T.
    ELSE
      s_actiontype = ""
      isRightField = .T.
    ENDIF


 OTHERWISE
 s_actiontype = ""
 isRightField = .T.

ENDCASE

IF isRightField AND s_actiontype # "BUTTON"
  CLEAR TYPEAHEAD
ENDIF



RETURN isRightField

*-------------------------------------------------------------
*
*
* FUNCTION brx_ord_valid
*
*   CALLED BY: FUNCTION browse_x           
*
*
*-------------------------------------------------------------
FUNCTION brx_ord_valid
PARAMETERS a_objects, n_order
n_lastkey = LASTKEY()

=say_say()

nSeekFld = 1
nTagName = 2
nObjName = 3
nUserDesc= 4
nBrowFld = 5
nFlag    = 6

*--flag this object's array row as true
a_objects[ n_order, nFlag ] = .T.

*--give the seek functions the field name they are comparing against
s_seek_fld  = a_objects[ n_order, nSeekFld ]

*--give the current controlling index/tag
s_cur_index = a_objects[ n_order, nTagName ]

*--the type of event
s_actiontype = "BUTTON"

*--clear out this old buffer && DO WE NEED THIS VAR ANYMORE?
s_old_seek = ''

**--IS THIS CODE DUPLICATED BY SUBSEQUENT F() CALL TO seek_it()?
n_recno = IIF(RECNO()>RECCOUNT(),RECCOUNT(),RECNO())
SET ORDER TO ( s_cur_index )
GO TOP
GO (n_recno)
READ TIMEOUT .001

=seek_it( s_seek_str, .T. )  & .T. = DO READ TIMEOUT .001 IN seek_it()

_curobj = OBJNUM( nGobrow )
KEYBOARD CHR(255)

RETURN
*---------------------------------------------------------
*
*
**    FUNCTION brx_rd_act     && Read Level Activate
*
*   CALLED BY: FUNCTION browse_x           
*
*---------------------------------------------------------
*
FUNCTION brx_rd_act     && Read Level Activate
*
* Activate Code from screen: BR
*
DO CASE
  CASE WLAST(s_wind_title) AND WEXIST(s_wind_title)
   SELECT Contact
  CASE !WEXIST(s_wind_title)
   CLEAR READ
ENDCASE

*---------------------------------------------------------
*
*
**    FUNCTION brx_rd_deact     && Read Level Deactivate
*
*   CALLED BY: FUNCTION browse_x           
*
*---------------------------------------------------------

FUNCTION brx_rd_deact     && Read Level Deactivate
*
* Deactivate Code from screen: BR
*
*
* The following IF statement gets executed
* when this screen is closed via clicking
* on the close box or 'Close' in the 'File' menu.
*
IF !WVISIBLE("BR_CONTROL") OR !WVISIBLE(s_wind_title)
   RELEASE WINDOWS br_control, (s_wind_title)
   l_quitting = .T.
   RETURN .T.
ENDIF	
*
* The following IF statement ensures
* that this application works
* stand-alone as well as when
* combined with a foundation read loop.
*
IF WONTOP(s_wind_title)
	IF RDLEVEL() > 1
		RETURN .T.
	ELSE
		RETURN .F.
	ENDIF
ENDIF
RETURN NOT WREAD()

**----------------------------------------------------------------
*
*
*        FUNCTION brx_rd_when
*
*
*   CALLED BY: FUNCTION browse_x           
*
**----------------------------------------------------------------
FUNCTION brx_rd_when

=map_keys(.T.)
=map_sel_keys(.T.)

RETURN

*-------------------------------------------------------------
*
*
*  FUNCTION map_sel_key
*
*   CALLED BY: FUNCTION browse_x           
*
*
*-------------------------------------------------------------
FUNCTION map_sel_key
PARAMETERS is_keys_on
IF is_keys_on
  ON KEY LABEL ctrl+Q DO brx_rd_kill WITH l_rd_kill, "ESC",   is_selected
*  ON KEY LABEL escape DO brx_rd_kill WITH l_rd_kill, "ESC",   is_selected
  ON KEY LABEL ctrl+W DO brx_rd_kill WITH l_rd_kill, "ENTER", is_selected
*  ON KEY LABEL enter  DO brx_rd_kill WITH l_rd_kill, "ENTER", is_selected
  ON KEY LABEL ctrl+end *
  ON KEY LABEL ctrl+home DO brx_FldWh WITH "HOTKEY"
ELSE
  ON KEY LABEL ctrl+Q
  ON KEY LABEL escape
  ON KEY LABEL ctrl+W
  ON KEY LABEL enter
  ON KEY LABEL ctrl+end
  ON KEY LABEL ctrl+home
ENDIF
RETURN
*-------------------------------------------------------------
*
*
*  PROCEDURE brx_rd_kill
*
*
*   CALLED BY: FUNCTION browse_x           
*
*-------------------------------------------------------------
PROCEDURE brx_rd_kill
PARAMETERS l_rd_kill, sKeyHit, is_selected

l_rd_kill = .T.
is_selected = IIF( sKeyHit=="ENTER",.T.,.F.)

CLEAR READ  && calls read valid routine instantaneously right here
=DATE() && a brings us back here to this line of code

KEYBOARD '{ESCAPE}'

RETURN
*-------------------------------------------------------------
*
*
*
*   FUNCTION say_say
*
*   CALLED BY: FUNCTION browse_x           
*
*-------------------------------------------------------------
FUNCTION say_say

 SHOW GET s_seek_str

 @ 0, WCOLS()-20 ;
   SAY PADL( a_objects[ n_order, nUserDesc ] + " Order", 20, " ") ;
   SIZE 2, 20 ;
   COLOR N/W

RETURN
**------------------------------------------------------------
*
* FUNCTION brx_rd_valid
*
*   CALLED BY: FUNCTION browse_x           
*
**------------------------------------------------------------
FUNCTION brx_rd_valid

IF INLIST( LASTKEY(), 27, 17, 23 )
  l_rd_kill = .T.
ENDIF


RETURN l_rd_kill
**------------------------------------------------------------
*
** FUNCTION seek_str_WHEN     &&  s_seek_str WHEN
*
*   CALLED BY: FUNCTION browse_x           
*
**------------------------------------------------------------
FUNCTION seek_str_WHEN     &&  s_seek_str WHEN

=map_keys(.F.)
=map_sel_keys(.F.)

l_hotkeys_off = .T.

RETURN
*------------------------------------------------------------
*
*     FUNCTION nGobrow_valid
*
*   CALLED BY: FUNCTION browse_x           
*
**------------------------------------------------------------
FUNCTION nGobrow_valid
KEYBOARD '{ctrl+f1}'
KEYBOARD '{end}'
KEYBOARD '{end}'
KEYBOARD '{end}'
KEYBOARD '{home}'

RETURN
*--------------------------------------------------------------
*
*            FUNCTION seek_func
*
*   CALLED BY: FUNCTION browse_x           
*
*--------------------------------------------------------------
FUNCTION seek_func
PARAMETER s_key_hit
PUSH KEY CLEAR

s_key_hit = UPPER( s_key_hit )
s_old_seek = s_seek_str 
n_old_rec  = RECNO()
n_loop = 0

IF SECONDS() - n_elaps_keytm > 2 AND s_key_hit # "BS"
  s_seek_str = ''
*  s_old_seek = ''
ENDIF

s_ikey_hit=''
DO WHILE .T.

*  n_ink_key = INKEY( .20, 'h' )
  n_ink_key = INKEY( .05, 'h' )

  IF BETWEEN( n_ink_key, 32, 126 ) 
    s_ikey_hit = UPPER( CHR( n_ink_key ) )
    s_seek_str = s_seek_str + STRTRAN( s_key_hit, "BS", "") + s_ikey_hit    
    s_key_hit = ''
  ENDIF

  IF "BS" $ s_key_hit && Backsp
    s_seek_str =  LEFT( s_old_seek, LEN( s_old_seek ) -1 )
    s_key_hit = ''
  ENDIF
  IF n_ink_key = 127
    s_seek_str = s_seek_str + s_key_hit
    s_seek_str =  LEFT( s_seek_str, LEN( s_seek_str ) -1 )
  ENDIF

  IF n_ink_key = 0 
    EXIT
  ENDIF   

  DO CASE
    CASE TYPE( s_seek_fld ) = "C"
      IF LEN( s_seek_str ) >= LEN( EVALUATE( s_seek_fld ))
	    s_seek_str = LEFT( s_seek_str, LEN( EVALUATE( s_seek_fld ))  )
      ENDIF
    CASE TYPE( s_seek_fld ) = "N"
       s_seek_str = LEFT( s_seek_str, 10  )
  ENDCASE   

  =say_say()

  n_loop = n_loop + 1
ENDDO


IF s_key_hit == "BS" && AND !EMPTY( s_old_seek )
  s_seek_str =  LEFT( s_old_seek, LEN( s_old_seek ) -1 )
ELSE
  s_seek_str = s_seek_str + s_key_hit
ENDIF
  
DO CASE
  CASE TYPE( s_seek_fld ) = "C"
    IF LEN( s_seek_str ) >= LEN( EVALUATE( s_seek_fld ))
       s_seek_str = LEFT( s_seek_str, LEN( EVALUATE( s_seek_fld ))  )
    ENDIF
  CASE TYPE( s_seek_fld ) = "N"
       s_seek_str = LEFT( s_seek_str, 10  )
ENDCASE   

SET NEAR ON
DO CASE
 CASE !EMPTY( s_seek_str ) AND TYPE ( s_seek_fld ) = "C"
   
   =seek_it( s_seek_str, .F. )  && for some reason this creates the 'ghost window' phenomenon
                            && when called from here with read timeout.  Strange FP bug


   *--store the record #
   n_recno = IIF(RECNO()>RECCOUNT(),RECCOUNT(),RECNO())  && RECNO()
   *--do refresh routine
   KEYBOARD '{alt+u}'
   KEYBOARD '{alt+u}'

 CASE !EMPTY( ALLTRIM( s_seek_str ) ) AND TYPE( s_seek_fld ) = "N"
   SEEK ( ALLTRIM( UPPER( s_seek_str)  ) )
   IF EOF()
*     SEEK ( ALLTRIM( UPPER( s_old_seek ) ) )
      GO BOTTOM
   ENDIF
   n_recno = IIF(RECNO()>RECCOUNT(),RECCOUNT(),RECNO())  && RECNO()
   KEYBOARD '{alt+u}'
   KEYBOARD '{alt+u}'
ENDCASE

 =say_say()

n_elaps_keytm = SECONDS()

SET NEAR OFF
POP KEY
RETURN 

*-----------------------------------------------------------------------
*
*  FUNCTION seek_it
*  
*
*-----------------------------------------------------------------------
FUNCTION seek_it
PARAMETERS s_seek_str, l_do_read_timeout
IF !EMPTY( s_seek_str )
  *----reseek
  SET NEAR ON
  SEEK ALLTRIM(s_seek_str)
   IF LEFT(s_seek_str,1) # LEFT(UPPER(EVAL( s_seek_fld )),1)
     s_seek_temp = CHR(ASC(LEFT(UPPER(LTRIM(s_seek_str)),1) )+1)
     SEEK (s_seek_temp)
     SKIP -1
     IF LEFT(s_seek_str,1) # LEFT(UPPER(EVAL( s_seek_fld )),1)
       SEEK ALLTRIM( s_seek_str )
       IF EOF()
         GO BOTTOM
       ENDIF
     ENDIF
   ENDIF
  SET NEAR OFF
  *--now, for some strange reason, a READ TIMEOUT .001 from
  *--within a browse window will cause a 'ghost window'
  *--I spent a lot of time waving dead chickens over this code
  *--before I figured out this was the culprit of the 'GHOST WINDOW'
  *--the 'ghost window' phenomenon is observed when 'ctrl+f1' is pressed
  *--to observe this phenomenon, change the =seek_it( s_seek_str, .F. ) 
  *--calls to =seek_it( s_seek_str, .T. ):: BUT REMEMBER, change 'em back!
  IF l_do_read_timeout  
    n_recno = IIF(RECNO()>RECCOUNT(),RECCOUNT(),RECNO())
    GO TOP
    GO (n_recno)
    READ TIMEOUT .001
  ENDIF
  *------------
ENDIF
RETURN
*-----------------------------------------------------------------------
*
*       PROCEDURE Go_rec
*
*
*   CALLED BY: FUNCTION browse_x           
*
*--------------------------------------------------------------
PROCEDURE Go_rec
GO TOP
 IF n_recno <= RECCOUNT()
   GO (n_recno )
 ENDIF
RETURN 
*--------------------------------------------------------------
*
*
*      FUNCTION map_keys
*
* 
*   CALLED BY: FUNCTION browse_x           
*
*--------------------------------------------------------------
FUNCTION map_keys
PARAMETER l_keys_on

IF l_keys_on
 ON KEY LABEL alt+u DO Go_rec
 ON KEY LABEL Backspace DO Seek_func WITH "BS"
 ON KEY LABEL spacebar DO Seek_func WITH " "
 ON KEY LABEL 1 DO Seek_func WITH "1"
 ON KEY LABEL 2 DO Seek_func WITH "2"
 ON KEY LABEL 3 DO Seek_func WITH "3"
 ON KEY LABEL 4 DO Seek_func WITH "4"
 ON KEY LABEL 5 DO Seek_func WITH "5"
 ON KEY LABEL 6 DO Seek_func WITH "6"
 ON KEY LABEL 7 DO Seek_func WITH "7"
 ON KEY LABEL 8 DO Seek_func WITH "8"
 ON KEY LABEL 9 DO Seek_func WITH "9"
 ON KEY LABEL 0 DO Seek_func WITH "0"
 ON KEY LABEL A DO Seek_func WITH "a"
 ON KEY LABEL B DO Seek_func WITH "b"
 ON KEY LABEL C DO Seek_func WITH "c"
 ON KEY LABEL D DO Seek_func WITH "d"
 ON KEY LABEL E DO Seek_func WITH "e"
 ON KEY LABEL F DO Seek_func WITH "f"
 ON KEY LABEL G DO Seek_func WITH "g"
 ON KEY LABEL H DO Seek_func WITH "h"
 ON KEY LABEL I DO Seek_func WITH "i"
 ON KEY LABEL J DO Seek_func WITH "j"
 ON KEY LABEL K DO Seek_func WITH "k"
 ON KEY LABEL L DO Seek_func WITH "l"
 ON KEY LABEL M DO Seek_func WITH "m"
 ON KEY LABEL N DO Seek_func WITH "n"
 ON KEY LABEL O DO Seek_func WITH "o"
 ON KEY LABEL P DO Seek_func WITH "p"
 ON KEY LABEL Q DO Seek_func WITH "q"
 ON KEY LABEL R DO Seek_func WITH "r"
 ON KEY LABEL S DO Seek_func WITH "s"
 ON KEY LABEL T DO Seek_func WITH "t"
 ON KEY LABEL U DO Seek_func WITH "u"
 ON KEY LABEL V DO Seek_func WITH "v"
 ON KEY LABEL W DO Seek_func WITH "w"
 ON KEY LABEL X DO Seek_func WITH "x"
 ON KEY LABEL Y DO Seek_func WITH "y"
 ON KEY LABEL Z DO Seek_func WITH "z"
 
 ON KEY LABEL CHR( 126) DO Seek_func WITH CHR( 126) && ~ 
 ON KEY LABEL CHR( 96 ) DO Seek_func WITH CHR( 96 ) && `
 ON KEY LABEL CHR( 33 ) DO Seek_func WITH CHR( 33 ) && !
 ON KEY LABEL CHR( 64 ) DO Seek_func WITH CHR( 64 ) && @
 ON KEY LABEL CHR( 35 ) DO Seek_func WITH CHR( 35 ) && #
 ON KEY LABEL CHR( 36 ) DO Seek_func WITH CHR( 36 ) && $
 ON KEY LABEL CHR( 37 ) DO Seek_func WITH CHR( 37 ) && %
 ON KEY LABEL CHR( 94 ) DO Seek_func WITH CHR( 94 ) && ^
 ON KEY LABEL CHR( 38 ) DO Seek_func WITH CHR( 38 ) && &
 ON KEY LABEL CHR( 42 ) DO Seek_func WITH CHR( 42 ) && *
 ON KEY LABEL CHR( 40 ) DO Seek_func WITH CHR( 40 ) && (
 ON KEY LABEL CHR( 41 ) DO Seek_func WITH CHR( 41 ) && )
 ON KEY LABEL CHR( 45 ) DO Seek_func WITH CHR( 45 ) && -
 ON KEY LABEL CHR( 95 ) DO Seek_func WITH CHR( 95 ) && _
 ON KEY LABEL CHR( 43 ) DO Seek_func WITH CHR( 43 ) && +
 ON KEY LABEL CHR( 61 ) DO Seek_func WITH CHR( 61 ) && =
 ON KEY LABEL CHR( 124) DO Seek_func WITH CHR( 124) && |
 ON KEY LABEL CHR( 92 ) DO Seek_func WITH CHR( 92 ) && \
 ON KEY LABEL LBRACE DO Seek_func WITH CHR( 123) && {
 ON KEY LABEL RBRACE DO Seek_func WITH CHR( 125) && }
 ON KEY LABEL CHR( 91 ) DO Seek_func WITH CHR( 91 ) && [
 ON KEY LABEL CHR( 93 ) DO Seek_func WITH CHR( 93 ) && ]
 ON KEY LABEL CHR( 58 ) DO Seek_func WITH CHR( 58 ) && :
 ON KEY LABEL CHR( 59 ) DO Seek_func WITH CHR( 59 ) && ;
 ON KEY LABEL CHR( 34 ) DO Seek_func WITH CHR( 34 ) && "
 ON KEY LABEL CHR( 39 ) DO Seek_func WITH CHR( 39 ) && '
 ON KEY LABEL CHR( 44 ) DO Seek_func WITH CHR( 44 ) && ,
 ON KEY LABEL CHR( 60 ) DO Seek_func WITH CHR( 60 ) && <
 ON KEY LABEL CHR( 46 ) DO Seek_func WITH CHR( 46 ) && .
 ON KEY LABEL CHR( 62 ) DO Seek_func WITH CHR( 62 ) && >
 ON KEY LABEL CHR( 63 ) DO Seek_func WITH CHR( 63 ) && ?
 ON KEY LABEL CHR( 47 ) DO Seek_func WITH CHR( 47 ) && /

ELSE
 ON KEY LABEL spacebar 
 ON KEY LABEL alt+u 
 ON KEY LABEL Backspace 
 ON KEY LABEL 1 
 ON KEY LABEL 2 
 ON KEY LABEL 3 
 ON KEY LABEL 4 
 ON KEY LABEL 5 
 ON KEY LABEL 6 
 ON KEY LABEL 7 
 ON KEY LABEL 8 
 ON KEY LABEL 9 
 ON KEY LABEL 0 
 ON KEY LABEL A 
 ON KEY LABEL B 
 ON KEY LABEL C 
 ON KEY LABEL D 
 ON KEY LABEL E 
 ON KEY LABEL F 
 ON KEY LABEL G 
 ON KEY LABEL H 
 ON KEY LABEL I 
 ON KEY LABEL J 
 ON KEY LABEL K 
 ON KEY LABEL L 
 ON KEY LABEL M 
 ON KEY LABEL N 
 ON KEY LABEL O 
 ON KEY LABEL P 
 ON KEY LABEL Q 
 ON KEY LABEL R 
 ON KEY LABEL S 
 ON KEY LABEL T 
 ON KEY LABEL U 
 ON KEY LABEL V 
 ON KEY LABEL W 
 ON KEY LABEL X 
 ON KEY LABEL Y 
 ON KEY LABEL Z 
 ON KEY LABEL CHR( 126) && ~ 
 ON KEY LABEL CHR( 96 ) && `
 ON KEY LABEL CHR( 33 ) && !
 ON KEY LABEL CHR( 64 ) && @
 ON KEY LABEL CHR( 35 ) && #
 ON KEY LABEL CHR( 36 ) && $
 ON KEY LABEL CHR( 37 ) && %
 ON KEY LABEL CHR( 94 ) && ^
 ON KEY LABEL CHR( 38 ) && &
 ON KEY LABEL CHR( 42 ) && *
 ON KEY LABEL CHR( 40 ) && (
 ON KEY LABEL CHR( 41 ) && )
 ON KEY LABEL CHR( 45 ) && -
 ON KEY LABEL CHR( 95 ) && _
 ON KEY LABEL CHR( 43 ) && +
 ON KEY LABEL CHR( 61 ) && =
 ON KEY LABEL CHR( 124) && |
 ON KEY LABEL CHR( 92 ) && \
 ON KEY LABEL LBRACE
 ON KEY LABEL RBRACE
 ON KEY LABEL CHR( 91 ) && [
 ON KEY LABEL CHR( 93 ) && ]
 ON KEY LABEL CHR( 58 ) && :
 ON KEY LABEL CHR( 59 ) && ;
 ON KEY LABEL CHR( 34 ) && "
 ON KEY LABEL CHR( 39 ) && '
 ON KEY LABEL CHR( 44 ) && ,
 ON KEY LABEL CHR( 60 ) && <
 ON KEY LABEL CHR( 46 ) && .
 ON KEY LABEL CHR( 62 ) && >
 ON KEY LABEL CHR( 63 ) && ?
 ON KEY LABEL CHR( 47 ) && /


 
ENDIF

RETURN
